﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;

namespace ODataVirtualizationDemo.Model
{
    public class Order : INotifyPropertyChanged, IEditableObject
    {
        #region Fields
        private int _orderId, _employeeId, _shipVia;
        private string _customerId, _shipName, _shipAddress, _shipCity,
            _shipRegion, _shipPostalCode, _shipCountry;
        private DateTimeOffset _orderDate, _requiredDate, _shippedDate;
        private decimal _freight;
        #endregion

        #region Properties

        public int OrderID
        {
            get => _orderId;
            set
            {
                if (_orderId != value)
                {
                    _orderId = value;
                    OnPropertyChanged();
                }
            }
        }

        public string CustomerID
        {
            get => _customerId;
            set
            {
                if (_customerId != value)
                {
                    _customerId = value;
                    OnPropertyChanged();
                }
            }
        }

        public int EmployeeID
        {
            get => _employeeId;
            set
            {
                if (_employeeId != value)
                {
                    _employeeId = value;
                    OnPropertyChanged();
                }
            }
        }

        public DateTimeOffset OrderDate
        {
            get => _orderDate;
            set
            {
                if (_orderDate != value)
                {
                    _orderDate = value;
                    OnPropertyChanged();
                }
            }
        }

        public DateTimeOffset RequiredDate
        {
            get => _requiredDate;
            set
            {
                if (_requiredDate != value)
                {
                    _requiredDate = value;
                    OnPropertyChanged();
                }
            }
        }

        public DateTimeOffset ShippedDate
        {
            get => _shippedDate;
            set
            {
                if (_shippedDate != value)
                {
                    _shippedDate = value;
                    OnPropertyChanged();
                }
            }
        }

        public int ShipVia 
        {
            get => _shipVia;
            set
            {
                if (_shipVia != value)
                {
                    _shipVia = value;
                    OnPropertyChanged();
                }
            }
        }

        public decimal Freight
        {
            get => _freight;
            set
            {
                if (_freight != value)
                {
                    _freight = value;
                    OnPropertyChanged();
                }
            }
        }

        public string ShipName 
        {
            get => _shipName;
            set
            {
                if (_shipName != value)
                {
                    _shipName = value;
                    OnPropertyChanged();
                }
            }
        }

        public string ShipAddress 
        {
            get => _shipAddress;
            set
            {
                if (_shipAddress != value)
                {
                    _shipAddress = value;
                    OnPropertyChanged();
                }
            }
        }

        public string ShipCity 
        {
            get => _shipCity; 
            set
            {
                if (_shipCity != value)
                {
                    _shipCity = value;
                    OnPropertyChanged();
                }
            }
        }

        public string ShipRegion
        {
            get => _shipRegion;
            set
            {
                if (_shipRegion != value)
                {
                    _shipRegion = value;
                    OnPropertyChanged();
                }
            }
        }

        public string ShipPostalCode 
        {
            get => _shipPostalCode;
            set
            {
                if (_shipPostalCode != value)
                {
                    _shipPostalCode = value;
                    OnPropertyChanged();
                }
            }
        }

        public string ShipCountry 
        {
            get => _shipCountry;
            set
            {
                if (_shipCountry != value)
                {
                    _shipCountry = value;
                    OnPropertyChanged();
                }
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            PropertyChanged?.Invoke(this, e);
        }

        #endregion

        #region IEditableObject Members

        private Order _clone;

        public void BeginEdit()
        {
            _clone = (Order)MemberwiseClone();
        }

        public void CancelEdit()
        {
            if (_clone != null)
            {
                foreach (var p in GetType().GetRuntimeProperties())
                {
                    if (p.CanRead && p.CanWrite)
                    {
                        p.SetValue(this, p.GetValue(_clone, null), null);
                    }
                }
            }
        }

        public void EndEdit()
        {
            _clone = null;
        }
        #endregion
    }
}
